// ListBoxWithExtras.cpp : implementation file
//
//This is the "meat and potatoes file.  The functionality of the
//enhanced listbox is right here.

#include "stdafx.h"
#include "ListBoxWithExtras.h"


#include <new.h>
#include <fstream.h>
namespace std {
#include <vector.h>
#include <algo.h>	//for sort() template
}

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// ListBoxWithExtras implementation

//Construction and Destruction
ListBoxWithExtras::ListBoxWithExtras()
: lbweInitialized(FALSE)
{}

ListBoxWithExtras::~ListBoxWithExtras() {}

//App Wizard sets this next stuff up.  This will trap the keydown
//and double click list box selection messages so we get a crack at them.
//If you construct your list box with the 'Notify' option unchecked (that's
//NOT the default, you won't get ListBoxWithExtras::EditEntry() functionality 
//when an string is double clicked.

BEGIN_MESSAGE_MAP(ListBoxWithExtras, CListBox)
	//{{AFX_MSG_MAP(ListBoxWithExtras)
	ON_CONTROL_REFLECT(LBN_DBLCLK, OnDblclk)
	ON_WM_KEYDOWN()
	ON_WM_PARENTNOTIFY()
	ON_WM_CHAR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

////////////////////////////////////////////////////////////////

void ListBoxWithExtras::AppendEntry()
//Adds a string to the box.  An empty string is shoved into the
//last slot and then we edit that blank box.  

{
	SetCurSel(InsertString(GetCount(), ""));
	CrankUpTheEdit(GetCount() - 1, INSERT_STRING);
}


void ListBoxWithExtras::InsertEntry()
//Inserts an entry before the highlighted entry.  If no entry
//is highlighted, the user gets a chance to add an entry to the 
//end of the list.

{
	int position = GetCurSel();

//If there are no highlighted entries, or no entries at all, 
//InsertEntry() becomes an AppendEntry()

	if (position == LB_ERR)
		AppendEntry();
	else
	{
		SetCurSel(InsertString (position, ""));
		CrankUpTheEdit(position, INSERT_STRING);
	}
}


void ListBoxWithExtras::EditEntry()
//Double clicking a listbox entry allows edit or it can be called
//from a program to operate on a highlighted selection

{
	int position = GetCurSel();
	if (position == LB_ERR)
		MessageBeep(MB_OK);
	else
		CrankUpTheEdit(position, EDIT_STRING);
}


void ListBoxWithExtras::DeleteEntry()
//Deletes the highlighted entry
{
	int position = GetCurSel();
	if (position == LB_ERR)  //entry highlighted?
		MessageBeep(MB_OK);
	else
	{
		ListBoxState undoBuff;
		GetText(position, undoBuff.valueLBS);  //save the to be deleted string
		SetCurSel(std::min(DeleteString(position) - 1, position));

		lbweBuffer.erase(lbweBuffer.begin() + position);
	//finish saving the previous state for Undo()
		undoBuff.positionLBS = position;
		undoBuff.actionLBS = DELETE_STRING;
		undoStack.push(undoBuff);
	}
}



void ListBoxWithExtras::Undo()
//Undo() "turns back the clock" and undoes the last
//change to the listbox

{
	if (undoStack.empty()) //anything to undo?
		MessageBeep(MB_OK);
	else
	{
		ListBoxState undo = undoStack.top();
		undoStack.pop();  //the STL stack pop() doesn't return a value		

		//Now we have all the info need to restore the previous state

		switch (undo.actionLBS)
		{
		case DELETE_STRING:
			SetCurSel(InsertString (undo.positionLBS, undo.valueLBS));
			lbweBuffer.insert(lbweBuffer.begin() + undo.positionLBS, undo.valueLBS);
			break;

		case INSERT_STRING:
			SetCurSel(std::min(DeleteString(undo.positionLBS) - 1, undo.positionLBS));
			lbweBuffer.erase(lbweBuffer.begin() + undo.positionLBS);
			break;

		case EDIT_STRING:
			DeleteString(undo.positionLBS);
			lbweBuffer.erase(lbweBuffer.begin() + undo.positionLBS);

			//When putting an entry into the box, the box style must be 
			//taken into account.  If it's a sorted box, CListBox::AddString()
			//does a fine job.

			int position;
			if (IsSortedBox())
				position = AddString(undo.valueLBS);
			else
				position = InsertString(undo.positionLBS, undo.valueLBS);
			SetCurSel(position);
			lbweBuffer.insert(lbweBuffer.begin() + position, undo.valueLBS);
			break;

		default:
			abort();  //we shouldn't ever get here
		}
	}
}



void ListBoxWithExtras::ResetUndo()
//ResetUndo() empties the undo stack.  If you do inserts and deletes
//through the inherited CListBox interface, call ResetUndo() if you want
//Undo() to work.  ListBoxWithExtras::SortList() also calls ResetUndo()
//since there's no easy way to undo a sort!

{
	CString temp;

	while (undoStack.size())
		undoStack.pop();
	lbweBuffer.erase(lbweBuffer.begin(), lbweBuffer.end());
	for (int i = 0; i < GetCount(); i++)
	{
		GetText(i, temp);
		lbweBuffer.push_back(temp);
	}

}


void ListBoxWithExtras::SortList()
{
//This is totally useless for a sorted box, but some users need sorted
//or unsorted for the same app.  So they can have their cake and eat it
//too.


	//First step is to save the currently highlighted entry so that it 
	//we can find and rehighlight it after the sort;

	CString currentHighlight;
	int position = GetCurSel();
	if (position != LB_ERR)
		GetText(position, currentHighlight);

	ResetContent();

//Ready to sort.  Unfortunately, the MFC CString class
//doesn't have an operator<() in a form useful for STL.  So we must
//create our own comparison function object and pass it to std::sort()
//See ListBoxWithExtras.h for the CompareCStrings function object 
//needed for the sort.

	std::sort (lbweBuffer.begin(), lbweBuffer.end(), CompareCStrings());

	//return sorted list to box;

	for (unsigned i = 0; i < lbweBuffer.size(); i++)
	{
		AddString(lbweBuffer[i]);

	//At same time, look for originally highlighted entry so that it can be re-highlighted
		if (position != LB_ERR && lbweBuffer[i] == currentHighlight)	
			SetCurSel(i);
	}
	ResetUndo();   //clear Undo() buffer.as SortList() would have scrambled it.
}


void ListBoxWithExtras::CrankUpTheEdit(int position, Action action)
//Private member that gets the edit box going.  First we save the
//current listbox state so that when the edit box self destructs and
//notifies the list box of same, we know what we're supposed to do
//with the edit box data.

//Note that CrankUpTheEdit() produces edit boxes which are consumed
//by the edit box handler.  Since the edit boxes commit suicide via
//delete this, the edit boxes MUST be created on the heap.  The edit
//destructor is private to enforce this restriction.

{
	//First, save the current state
	GetText(position, currentState.valueLBS);
	currentState.boxInFocus = GetFocus();
	currentState.positionLBS = position;
	currentState.actionLBS = action;

	//Now fire up the edit box.
	RECT rect;
	GetItemRect(position, &rect);
	pEditBox = new EditBox (rect, currentState.valueLBS, this, GetFont());
}


inline
BOOL ListBoxWithExtras::IsSortedBox() 
//Tells whether or not the box is sorted

{
	return GetWindowLong(m_hWnd, GWL_STYLE) & LBS_SORT;
}

/////////////////////////////////////////////////////////////////////////////
// ListBoxWithExtras message handlers


void ListBoxWithExtras::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 

//Trap key presses in order to integrate <Delete> and <Insert> into the
//control.  We're also changing <End> to mean AppendEntry()

{
	switch (nChar)
	{
	case VK_END:	//<End>
		AppendEntry();
		return;
	case VK_DELETE:	//<Delete>
		DeleteEntry();
		return;
	case VK_INSERT:	//<Insert>
		InsertEntry();
		return;
	}

	//Just pass along keys you don't care about.
	CListBox::OnKeyDown(nChar, nRepCnt, nFlags);
}


void ListBoxWithExtras::OnDblclk() 
//Double clicking a listbox entry gets us here
{
	ASSERT(LB_ERR != GetCurSel());
	EditEntry();
}


void ListBoxWithExtras::OnParentNotify(UINT message, LPARAM lParam) 
//Here's where the edit box and the list box do their handshaking.
//When the edit box is about to be destroyed, it notifies its list box
//parent of its impending doom and OnParentNotify grabs the pertinent
//data from the edit box before the box commits suicide.

{
	CListBox::OnParentNotify(message, lParam);
	
	if (LOWORD(message) == WM_DESTROY) //edit box calls DestroyWindow()?
	{
		currentState.boxInFocus->SetFocus(); //restore focus to where it was b4 edit box

		switch (currentState.actionLBS)
		{
		case INSERT_STRING:
			DeleteString(currentState.positionLBS);
			break;
		case EDIT_STRING:
			if (pEditBox->TransferOK())
			{
				DeleteString(currentState.positionLBS);
				lbweBuffer.erase(lbweBuffer.begin() + currentState.positionLBS);
			}
			break;
		default:
			abort();
		}

		if (pEditBox->TransferOK())  //User pressed <Enter> to close edit box?
		{
			CString ebReturn;
			pEditBox->GetWindowText(ebReturn);
			if (!IsSortedBox())
				InsertString(currentState.positionLBS, ebReturn);
			else
				currentState.positionLBS = AddString(ebReturn);
			lbweBuffer.insert(lbweBuffer.begin() + currentState.positionLBS, ebReturn);

			//Transfer complete.  Now save the changes that were made
			//into the undo stack in case we want to reverse the change.

			undoStack.push(currentState);
		}
	}
	SetCurSel(currentState.positionLBS);
}


void ListBoxWithExtras::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 

//Here's the handler needed for calling member functions from 
//"regular keys".  <Ctrl>-Y will delete an entry and <Ctrl>-U will
//invoke ListBoxWithExtras::Undo()

{
	const UINT CtrlY = 25;	//delete command
	const UINT CtrlU = 21;	//undo command

	switch (nChar)
	{
	case CtrlU:
		Undo();
		return;
	
	case CtrlY:
		DeleteEntry();
		return;
	}
	
	//all other chars handles by base class
	CListBox::OnChar(nChar, nRepCnt, nFlags);
}

LRESULT ListBoxWithExtras::DefWindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if (!lbweInitialized)
	{
		CString temp;

		lbweInitialized = TRUE;
		std::vector<CString>::iterator i;
		for (i = lbweBuffer.begin(); i != lbweBuffer.end(); i++)
			AddString(*i);
		lbweBuffer.erase(lbweBuffer.begin(), lbweBuffer.end());
		for (int j = 0; j < GetCount(); j++)
		{
			GetText(j, temp);
			lbweBuffer.push_back(temp);
		}
		SetCurSel(0);
	}

	return CListBox::DefWindowProc(message, wParam, lParam);

}






